%% Identifies the nominal plant model G based on the frequency response data

close all
clear
clc

h=figure(9);

%% Simulation variables

Dt_step = 4e-5;


%%  -- System identification --
bodeOpt = bodeoptions;
bodeOpt.FreqUnits = 'Hz';
bodeOpt.PhaseMatching = 'on';
bodeOpt.PhaseMatchingFreq = 100;
bodeOpt.PhaseMatchingValue = 0;
bodeOpt.YLim = {[-80,25];[-360,0]}; % {maglimits; phaselimits}
bodeOpt.YLimMode = {'manual';'auto'}; 

%% Time delay
load('data/01 - Plant Response G_hat/Run 1/GXX.MAT');

f = gxx(5:end,1);
d = gxx(5:end,2);
p = phase(gxx(5:end,2));

Gxx_experimental = frd(d,f,'FrequencyUnit','Hz');

% Time delay
l = fit(f(5:156),p(5:156),'poly1');

plot(l,f,p);
xlabel('Frequency [hz]'); ylabel('Phase [rad]');
grid on

export_plot('experiment_03_MIMO_Gxx_delay.pdf');

Td = -l.p1/(2*pi);


%% Gxx
np = 3;
nz = 2;

wpass = [20,140; 358,370; 540,548;  666,672; 792,794; 834,846]*2*pi;
opt = tfestOptions('Focus',wpass);
Gxx = tfest(Gxx_experimental,np,nz,Td,opt);

bode(Gxx_experimental,Gxx,bodeOpt);
childHnd=get(h, 'Children'); axes1=childHnd(3);
legend1 = legend(axes1,'$\hat{G}$', '$G$','Location','NorthWest');
set(legend1,'Interpreter','latex');
title('');
xlim([20 3200]);
grid on

export_plot('experiment_03_MIMO_Gxx_id.pdf');

%% Gyy
load('data/01 - Plant Response G_hat/Run 1/GYY.MAT');

f = gyy(5:end,1);
d = gyy(5:end,2);
p = phase(gyy(5:end,2));

Gyy_experimental = frd(d,f,'FrequencyUnit','Hz');


% Use same time delay as from Gxx: Td

% System identification
np = 3;
nz = 2;

wpass = [20,140; 358,370; 540,548;  666,672; 752,821; 1100,1108]*2*pi;
opt = tfestOptions('Focus',wpass);
Gyy = tfest(Gyy_experimental,np,nz,Td,opt);

bode(Gyy_experimental,Gyy,bodeOpt);
childHnd=get(gcf, 'Children'); axes1=childHnd(3);
legend1 = legend(axes1,'$\hat{G}$', '$G$','Location','SouthWest');
set(legend1,'Interpreter','latex');
title('');
xlim([20 3200]);
grid on

export_plot('experiment_03_MIMO_Gyy_id.pdf');



%% Gxy
load('data/01 - Plant Response G_hat/Run 1/GXY.MAT');

f = gxy(5:end,1);
d = gxy(5:end,2);
p = phase(gxy(5:end,2));

Gxy_experimental = frd(d,f,'FrequencyUnit','Hz');

% Use same time delay as from Gxx: Td

% System identification
np = 2;
nz = 2;

wpass = [10,300; 756,760;]*2*pi;
opt = tfestOptions('Focus',wpass);
Gxy = tfest(Gxy_experimental,np,nz,Td,opt);

bode(Gxy_experimental,Gxy,bodeOpt);
childHnd=get(gcf, 'Children'); axes1=childHnd(3);
legend1 = legend(axes1,'$\hat{G}$', '$G$','Location','NorthWest');
set(legend1,'Interpreter','latex');
title('');
xlim([20 3200]);
grid on

export_plot('experiment_03_MIMO_Gxy_id.pdf');


%% Gyx
load('data/01 - Plant Response G_hat/Run 1/GYX.MAT');

f = gyx(5:end,1);
d = gyx(5:end,2);
p = phase(gyx(5:end,2));

Gyx_experimental = frd(d,f,'FrequencyUnit','Hz');

% Use same time delay as from Gxx: Td

% System identification
np = 2;
nz = 2;

wpass = [10,300; 768,772;]*2*pi;
opt = tfestOptions('Focus',wpass);
Gyx = tfest(Gyx_experimental,np,nz,Td,opt);

bode(Gyx_experimental,Gyx,bodeOpt);
childHnd=get(gcf, 'Children'); axes1=childHnd(3);
legend1 = legend(axes1,'$\hat{G}$', '$G$','Location','NorthWest');
set(legend1,'Interpreter','latex');
title('');
xlim([20 3200]);
grid on

export_plot('experiment_03_MIMO_Gyx_id.pdf');


%% Wrap it up

G = [Gxx, Gxy; Gyx, Gyy];
G_nodelay = pade(tf(G),0);

