%% Generates several of the plots used in the thesis, using data from the experiments

ControllerName = '04 - Hinf MIMO'; % 02 - PID :: 03 - Hinf SISO :: 04 - Hinf MIMO
ControllerID = 'Hinf_MIMO';        % PID      :: Hinf_SISO      :: Hinf_MIMO

all_freq = [10 50]; %[1 10 20 50 100 200 300 400 662 760];

for ref_freq = all_freq;
num_periods = 9;

load(['data\',ControllerName,'\Scanning\data_',int2str(ref_freq),'hz.mat'], 'td', 'xd', 'yd');
NumSamplesPerHalfPeriod = (1/ref_freq)*1/(2*Dt_step);
NumSamples = round(2*NumSamplesPerHalfPeriod*num_periods);
x_offset = sum(xd(1:NumSamples,1),1)/NumSamples;
exd = xd(:,2)-xd(:,1);
y_offset = sum(yd(1:NumSamples,1),1)/NumSamples;
eyd = yd(:,2)-yd(:,1);


SamplesForStd = round(NumSamplesPerHalfPeriod*0.2):round(NumSamplesPerHalfPeriod*0.8);
for i=1:num_periods; SamplesForStd = [SamplesForStd round(SamplesForStd+2*NumSamplesPerHalfPeriod)]; end;
fprintf('Standard deviation (%s, %dhz): x = %f  y = %f\n', ControllerID, ref_freq,...
    std(exd(SamplesForStd))*1000, std(eyd(SamplesForStd))*1000 );

% x
plot(td,xd-x_offset,td,exd,'r');

legend('Reference r(t)','Response y_m(t)','Error e(t)');
xlabel('Time (s)');
ylabel('Displacement (m)');
xlim([0 td(NumSamples)]);
ylim([-1.1 1.1]);
title('Displacement along x-axis');
grid on
export_plot(['experiment_03_MIMO_',ControllerID,'_Scan_',int2str(ref_freq),'hz_x.pdf']);

% y
plot(td,yd-y_offset,td,eyd,'r');

legend('Reference r(t)','Response y_m(t)','Error e(t)');
xlabel('Time (s)');
ylabel('Displacement (m)');
title('Displacement along y-axis');
xlim([0 td(NumSamples)]);
ylim([-1.1 1.1]);
grid on
export_plot(['experiment_03_MIMO_',ControllerID,'_Scan_',int2str(ref_freq),'hz_y.pdf']);


% xy

plot(xd(1:2*NumSamplesPerHalfPeriod*num_periods,:)-x_offset,yd(1:2*NumSamplesPerHalfPeriod*num_periods,:)-y_offset);
set(gca,'PlotBoxAspectRatio',[1 1 1]);
grid on;
xlabel('Displacement x [m]'); ylabel('Displacement y [m]');
xlim([-1.1 1.1]);
ylim([-.55 .55]);
export_plot(['experiment_03_MIMO_',ControllerID,'_Scan_',int2str(ref_freq),'hz_xy.pdf']);

end



%% Plot complementary sensitivity function

%% Load data

load('data\02 - PID\T\TXX.mat');
load('data\02 - PID\T\TXY.mat');
load('data\02 - PID\T\TYX.mat');
load('data\02 - PID\T\TYY.mat');

txx = frd(txx(5:end,2), txx(5:end,1),'FrequencyUnit','Hz');
txy = frd(txy(5:end,2), txy(5:end,1),'FrequencyUnit','Hz');
tyx = frd(tyx(5:end,2), tyx(5:end,1),'FrequencyUnit','Hz');
tyy = frd(tyy(5:end,2), tyy(5:end,1),'FrequencyUnit','Hz');

T_experimental_PID = [txx, txy; tyx, tyy];

load('data\03 - Hinf SISO\T\TXX.mat');
load('data\03 - Hinf SISO\T\TXY.mat');
load('data\03 - Hinf SISO\T\TYX.mat');
load('data\03 - Hinf SISO\T\TYY.mat');

txx = frd(txx(5:end,2), txx(5:end,1),'FrequencyUnit','Hz');
txy = frd(txy(5:end,2), txy(5:end,1),'FrequencyUnit','Hz');
tyx = frd(tyx(5:end,2), tyx(5:end,1),'FrequencyUnit','Hz');
tyy = frd(tyy(5:end,2), tyy(5:end,1),'FrequencyUnit','Hz');

T_experimental_Hinf_SISO = [txx, txy; tyx, tyy];

load('data\04 - Hinf MIMO\T\TXX.mat');
load('data\04 - Hinf MIMO\T\TXY.mat');
load('data\04 - Hinf MIMO\T\TYX.mat');
load('data\04 - Hinf MIMO\T\TYY.mat');

txx = frd(txx(5:end,2), txx(5:end,1),'FrequencyUnit','Hz');
txy = frd(txy(5:end,2), txy(5:end,1),'FrequencyUnit','Hz');
tyx = frd(tyx(5:end,2), tyx(5:end,1),'FrequencyUnit','Hz');
tyy = frd(tyy(5:end,2), tyy(5:end,1),'FrequencyUnit','Hz');

T_experimental_Hinf_MIMO = [txx, txy; tyx, tyy];

%% Plot comparison Hinf vs PID

bode(T_experimental_Hinf_MIMO(1,1),T_experimental_Hinf_SISO(1,1),T_experimental_PID(1,1));
title('$\hat{T}_{xx}$','Interpreter','latex');
xlim([20 3200]);
legend1 = legend('${H_\infty}$ MIMO','${H_\infty}$ SISO','${PID}$','Location','SouthWest');
set(legend1,'Interpreter','latex');
grid on
export_plot('experiment_03_MIMO_T_hat_Hinf_vs_PID_xx.pdf');

bode(T_experimental_Hinf_MIMO(1,2),T_experimental_Hinf_SISO(1,2),T_experimental_PID(1,2));
title('$\hat{T}_{xy}$','Interpreter','latex');
xlim([20 3200]);
legend1 = legend('${H_\infty}$ MIMO','${H_\infty}$ SISO','${PID}$','Location','SouthWest');
set(legend1,'Interpreter','latex');
grid on
export_plot('experiment_03_MIMO_T_hat_Hinf_vs_PID_xy.pdf');
%%
bode(T_experimental_Hinf_MIMO(2,1),T_experimental_Hinf_SISO(2,1),T_experimental_PID(2,1));
title('$\hat{T}_{yx}$','Interpreter','latex');
xlim([20 3200]);
legend1 = legend('${H_\infty}$ MIMO','${H_\infty}$ SISO','${PID}$','Location','SouthWest');
set(legend1,'Interpreter','latex');
grid on
export_plot('experiment_03_MIMO_T_hat_Hinf_vs_PID_yx.pdf');
%%
bode(T_experimental_Hinf_MIMO(2,2),T_experimental_Hinf_SISO(2,2),T_experimental_PID(2,2));
title('$\hat{T}_{yy}$','Interpreter','latex');
xlim([20 3200]);
legend1 = legend('${H_\infty}$ MIMO','${H_\infty}$ SISO','${PID}$','Location','SouthWest');
set(legend1,'Interpreter','latex');
grid on
export_plot('experiment_03_MIMO_T_hat_Hinf_vs_PID_yy.pdf');


%% Hinf MIMO controller




hb = sigmaplot(T_experimental_Hinf_MIMO, T);
setoptions(hb,'FreqUnits','Hz');
legend1 = legend('$\hat{T}_{H\infty}$','$T_{H\infty}$','Location','SouthWest');
set(legend1,'Interpreter','latex');
title('H_\infty MIMO');
grid on
xlim([20 3200]);
ylim([-60 0]);

export_plot('experiment_03_MIMO_Hinf_MIMO_T_vs_T_hat.pdf');

%% Hinf SISO controller




hb = sigmaplot(T_experimental_Hinf_SISO, T_SISO);
setoptions(hb,'FreqUnits','Hz');
legend1 = legend('$\hat{T}_{H\infty}$','$T_{H\infty}$','Location','SouthWest');
set(legend1,'Interpreter','latex');
title('H_\infty SISO');
grid on
xlim([20 3200]);
ylim([-60 0]);

export_plot('experiment_03_MIMO_Hinf_SISO_T_vs_T_hat.pdf');


%% PID controller




hb = sigmaplot(T_experimental_PID, T_PID);
setoptions(hb,'FreqUnits','Hz');
legend1 = legend('$\hat{T}_{PID}$','$T_{PID}$','Location','SouthWest');
set(legend1,'Interpreter','latex');
title('PID');
xlim([20 3200]);
ylim([-60 0]);
grid on



export_plot('experiment_03_MIMO_PID_T_vs_T_hat.pdf');





