%Finds the maximum stable step-size for a given controller and generates
%the plot of the stability regions for several explicit Runge-Kutta
%solvers.

% Assumes controller K is already generated
for k_r_orders = 10:17
	% Create the reduced controller
	new_order = k_r_orders;
	[K_bal,g] = balreal(K);
	elim = [ zeros(new_order,1); ones(size(g,1)-new_order,1) ];
	K_r = modred(K_bal,logical(elim));

	K_eig = sort(eig(K),'descend');
	K_r_eig = sort(eig(K_r),'descend');

	% Binary search to find the maximum step-size
	erk_orders = [1:5,8];
	max_step_sizes = zeros(size(erk_orders));
	
	for i = 1:size(erk_orders,2)
		low = 0;
		high = 1;
		epsilon = 1e-9;

		while (high - low ) > epsilon
		   h = (high - low) / 2 + low;
		   z = h * K_eig;
		   if abs( Phi(z,erk_orders(i)) ) <= 1
			   low = h;
		   else
			   high = h;
		   end
		end

		max_step_sizes(1,i) = low;
	end

	display(regexprep( num2str(max_step_sizes*1e6,4), '([0-9])\s+([0-9])', '$1\t$2'))
end
%% plot
[re, im] = meshgrid(-6.0:.01:3, -6:.01:6);
z = complex(re,im);

ineq = zeros(size(z));
figure(9); hold off;

for i=erk_orders
    ineq = ineq + (ineq | (abs(Phi(z,i))<=1)); 
end

[C,h] = contourf(re,im, ineq, [1,2,3,4,5,5.5]) ;

grid on;
xlabel('Re($z$)','Interpreter','latex');
ylabel('Im($z$)','Interpreter','latex');

hold on;
re = real(K_eig*max_step_sizes(end));
im = imag(K_eig*max_step_sizes(end));
plot(re,im,'om');
re = real(K_r_eig*max_step_sizes(end));
im = imag(K_r_eig*max_step_sizes(end));
plot(re,im,'xy');
daspect([1 1 1])
hold off

export_plot('experiment_07_rk_stability_K_18_eigenvalues.pdf');
