%% This code runs the AFM platform using the simulink model controller_xpc 
% with a raster pattern reference signal at the frequencies specified in
% all_freq, then saves the data and plots the results.

% Initialize
tg = xpc('TargetPC1');
tg.load('controller_xpc');

all_freq = [1 10 20 50 100 200 300 400 662 760];

for ScanRate=all_freq

LengthPerVolt = 4.664678428174072; % um/V
Offset = [7.4 6.8];

setparam(tg, getparamid(tg, 'Offset_XY', 'Value'), Offset); % V

ScanWidth = 2;  % um
NumLines = 10 - 5*(ScanRate < 10);

LineLength = ScanWidth / NumLines;
ScanSpeedY = LineLength * ScanRate; % um/s
ScanWidthV = ScanWidth / LengthPerVolt; % V
ScanRampSlopeYV = ScanSpeedY / LengthPerVolt; % V
SimTime = ScanWidth / ScanSpeedY + 1/ScanRate;

setparam(tg, getparamid(tg, 'Step_Ref', 'Time'), 1/ScanRate); % Delay scanning by one period to stabilize
setparam(tg, getparamid(tg, 'Sequence_X/Constant', 'Value'), 1/ScanRate); % Scan Period
setparam(tg, getparamid(tg, 'Sequence_X/Look-Up Table1', 'InputValues'), [0 .5 1]/ScanRate); % Time
setparam(tg, getparamid(tg, 'Sequence_X/Look-Up Table1', 'Table'), [0 ScanWidthV 0]); % Output

setparam(tg, getparamid(tg, 'Ramp_Y/Step', 'After'), ScanRampSlopeYV );

set(tg, 'StopTime', SimTime);

disp( ['Simulation time: ', num2str(SimTime), ' s'] );

tg.start();

while( ~strcmp(tg.status, 'stopped') )
   pause(0.25);
end


% Plot

start_index = floor((1/ScanRate)/Dt_step);

td = tg.TimeLog(start_index:end)-tg.TimeLog(start_index);
xd = tg.OutputLog(start_index:end,[1 3]) * LengthPerVolt; % um
yd = tg.OutputLog(start_index:end,[2 4]) * LengthPerVolt; % Assuming um/V value is same as for x (should check this)
exd = xd(:,2)-xd(:,1) + sum(xd(:,2),1)/size(xd,1);
eyd = yd(:,2)-yd(:,1) + sum(yd(:,2),1)/size(yd,1);

%save(['..\02 - PID\Scanning\data_',int2str(ScanRate),'hz.mat'], 'td', 'xd', 'yd');
%save(['..\03 - Hinf SISO\Scanning\data_',int2str(ScanRate),'hz.mat'], 'td', 'xd', 'yd');
%save(['..\04 - Hinf MIMO\Scanning\data_',int2str(ScanRate),'hz.mat'], 'td', 'xd', 'yd');


plot(td,xd,td,yd,td,exd,'r',td,eyd);

plot(xd,yd);
set(gca,'PlotBoxAspectRatio',[1 1 1]);
grid on;
xlabel('Displacement x [um]'); ylabel('Displacement y [um]');

std(exd)
std(eyd)

end